// @(#)root/base:$Id: TObject.h 22961 2008-04-03 13:32:57Z rdm $
// Author: Fons Rademakers   26/06/2008

/*************************************************************************
 * Copyright (C) 1995-2008, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifndef ROOT_TMemoryPool
#define ROOT_TMemoryPool

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TMemoryPool                                                          //
//                                                                      //
// Try to avoid memory fragmentation by allocating objects in a         //
// large memory pool. Also allocate memory in mmapped area to avoid     //
// extra memory copies from user to kernel space when reading/writing   //
// I/O buffers.                                                         //
//                                                                      //                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TObject
#include "TObject.h"
#endif


class TMemoryPool : public TObject {

private:
   char   *fMemory;  // pointer to pools memory
   UInt_t  fSize;    // size of the pool
   UInt_t  fPos;     // offset of first unallocated byte
   UInt_t  fEnd;     // offset of last unallocated byte
   Bool_t  fMmap;    // true if mmapped, else malloc'ed

   TMemoryPool(const TMemoryPool &pool);  // not implemented
   void operator=(const TMemoryPool &);   // not implemented

public:
   TMemoryPool(UInt_t max, Bool_t useMmap = kTRUE);
   virtual ~TMemoryPool();

   void  *Allocate(UInt_t size, Bool_t fromEnd = kFALSE);
   void  *ReAllocate(void *old, UInt_t oldSize, UInt_t newSize);

   void  *Reset(void *keep = 0, UInt_t size = 0);

   void   Print(Option_t *option="") const;

   ClassDef(TMemoryPool,0)  // Memory pool class
};

#endif
